/**
 * <p>File: DifferentiableFunction.java</p>
 * <p>Title: Numerical methods</p>
 * <p>Description: Course on numerical analysis</p>
 * <p>Date: January 2016</p>
 * @author Faton M. Berisha
 * @version 6.0.0
 */

package numeric;

import java.util.Observer;

/**
 * An abstract solver.
 */
public abstract class Solver {
	protected Matrix solution;

	private double tolerance = 1E-16;
	private int numIterations = 50;

	/**
	 * Initializes an abstract solver in <code>n</code> variables.
	 * 
	 * @param n
	 *            the number of variables
	 */
	public Solver(int n) {
		solution = new Matrix(n, 1);
	}

	/**
	 * Gets the previously calculated solution.
	 * 
	 * @return <code>Matrix</code> containing the previously calculated
	 *         solution
	 */
	public Matrix getSolution() {
		Matrix answer = null;
		try {
			answer = (Matrix) solution.clone();
		} catch (CloneNotSupportedException e) {
		}
		return answer;
	}

	/**
	 * Adds a solution observer to this solver.
	 * 
	 * @param observer
	 *            the <code>java.util.Observer</code> observer to be added
	 */
	public void addSolutionObserver(Observer observer) {
		solution.setDemo(true);
		solution.addObserver(observer);
	}

	/**
	 * Gets this solver's error tolerance.
	 * 
	 * @return solution approximating tolerance
	 */
	public double getTolerance() {
		return tolerance;
	}

	/**
	 * Sets this solver's error tolerance.
	 * 
	 * @param tolerance
	 *            tolerance value
	 */
	public void setTolerance(double tolerance) {
		this.tolerance = tolerance;
	}

	/**
	 * Gets the maximum number of iterations for this solver's iterative
	 * methods.
	 * 
	 * @return the maximum number of iterations
	 */
	public int getNumIterations() {
		return numIterations;
	}

	/**
	 * Sets the maximum number of iterations for this solver's iterative
	 * methods.
	 * 
	 * @param tolerance
	 *            specifies the maximum number of iterations
	 */
	public void setNumIterations(int numIterations) {
		this.numIterations = numIterations;
	}

	/**
	 * Gets the number of equations.
	 * 
	 * @return number of equations
	 */
	public abstract int getNumEquations();

	/**
	 * Gets the number of variables.
	 * 
	 * @return number of variables
	 */
	public int getNumVariables() {
		return solution.getNumRows();
	}
}
